import { Axios, AxiosError } from "axios";
import { Credentials, Token, User, Flight, FlightCreate } from "./Types";

const auth_instance = new Axios({
	baseURL: "http://127.0.0.1:5001/",
	headers: {
		accept: "application/json",
		"Content-Type": "application/json",
	},
    validateStatus: (x) => { return !(x < 200 || x > 204) }
});

const flights_instance = new Axios({
	baseURL: "http://127.0.0.1:5000/",
	headers: {
		accept: "application/json",
		"Content-Type": "application/json",
	},
    validateStatus: (x) => { return !(x < 200 || x > 204) }
});

auth_instance.interceptors.response.use(
	(response) => {
		return JSON.parse(response.data);
	},
	(error) => {
		const err = error as AxiosError;
		return Promise.reject(err);
	}
);

flights_instance.interceptors.request.use((request) => {
	request.data = JSON.stringify(request.data);
	return request;
});

flights_instance.interceptors.response.use(
	(response) => {
		return JSON.parse(response.data);
	},
	(error) => {
		const err = error as AxiosError;
		return Promise.reject(err);
	}
);

auth_instance.interceptors.request.use((request) => {
	request.data = JSON.stringify(request.data);
	return request;
});

export const createUser = (
	credentials: Credentials
): Promise<{ id?: string; message: string }> => {
	return auth_instance.post("users", credentials);
};

export const fetchUsers = (): Promise<User[]> => {
	return auth_instance.get("users");
};

export const fetchUserById = (id: number): Promise<User> => {
	return auth_instance.get("users/" + id);
};

export const logIn = (
	credentials: Credentials
): Promise<Token & Partial<{ message: string; user_id: number }>> => {
	return auth_instance.post("auth/login", credentials);
};

export const tokenStatus = (
	token: string
): Promise<User & { message?: string }> => {
	return auth_instance.get("auth/status", {
		headers: { Authorization: `Bearer ${token}` },
	});
};

export const fetchZones = (origin: string | null): Promise<Flight[]> => {
    return flights_instance.get("flights" + (origin ? "?origin=" + origin : ""))
};

export const createFlight = (
	flight_data: FlightCreate
): Promise<Flight> => {
	return flights_instance.post("flights", flight_data);
};